#!perl -w

# -----------------------------------------------------------------------------
# Program: demonstration of CGI-technology for solving DEA problems
# Author: Eugene P. Morgunov
# Date:  June 25, 2005
# Version: 1.0
# -----------------------------------------------------------------------------

# NOTE. We use famous DEAP program in this demo. DEAP serves as a DEA solver.

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );

my $query;                       # CGI-object

my $data_file;                   # data file name
my $instr_file;                  # instructions file name
my $output_file;                 # output file name

my $data;                        # data file contents
my $instr;                       # instructions file contents
my @instr_array;                 # array for storing instructions

$query = CGI->new();             # create a new CGI-object

# get contents of two windows into variables
$data = $query->param( 'data' );
$instr = $query->param( 'instr' );

# get instr file name
$instr_file = $query->param( 'instr_file_name' );

# remove spare '\r' symbols from the strings
# NOTE. Where are these '\r' symbols from ?
$data =~ s/\r//mg;      # m - means 'multiline'
$instr =~ s/\r//mg;

# split instructions string into separate lines
@instr_array = split( /\n/, $instr );

# get file names from the first two lines of the instructions array
( $data_file ) = $instr_array[ 0 ] =~ /^\s*(.+?)\s+/;
( $output_file ) = $instr_array[ 1 ] =~ /^\s*(.+?)\s+/;

# turn off buffering of devices
select( STDERR ); 
$| = 1;
select( STDOUT );
$| = 1;

# create HTTP-header
print "Content-type: text/html\n\n";

# starting HTML-tags
print "<HTML>\n";
print "<HEAD>\n";
print "<TITLE>Demonstration of CGI-technology for solving DEA problems";
print "</TITLE>\n";
print "</HEAD>\n";

# white background
# NOTE. The first two hexadecimal digits correspond to brightness of
#       red colour, the next two -- of green colour, the last two -- of blue
#       colour. Values of these digits are from 0 to F.
print "<BODY BGCOLOR=\"#FFFFFF\">\n";

# DEBUG
#print "$instr_array[ 0 ]\n <BR>";
#print "$instr_array[ 1 ]\n <BR>";

# DEBUG
#print "$data_file\n <BR>";
#print "$instr_file\n <BR>";


# data file
open( DATA, "> $data_file" ) ||
      die "Can't open data file $data_file: $!\n";

open( INSTR, "> $instr_file" ) ||
      die "Can't open instructions file $instr_file: $!\n";

# store contents of screen windows into files
print( DATA $data );
print( INSTR $instr );

# this message is for a user's information
# DEBUG
#print "Data and instructions are stored into files\n";

# close our files
close( DATA ) || 
       die "Can't close data file $data_file: $!\n";

close( INSTR ) || 
       die "Can't close instructions file $instr_file: $!\n";

# ----------------------------------------------
# make the main job
system( "echo $instr_file | deap.exe > nul" );
# ----------------------------------------------

# now we must display output file contents in a browser's window
open( OUTPUT, "< $output_file" ) ||
      die "Can't open output file $output_file: $!\n";

print "<PRE>\n";

# read output file line by line
while ( <OUTPUT> )
{
  chomp;                 # remove LF/CR symbols
  print $_ . "\n";       # print one line of a file
}

close( OUTPUT ) || 
       die "Can't close output file $output_file: $!\n";


# create closing HTML-tags
print "</BODY>\n";
print "</HTML>\n";

# quitting the program
exit( 0 );
